#/bin/sh -x

if [ $# -eq 0 ]; then
    echo "CamReport <report_name> [-today] [[-i] <rollname>]"
    echo "Where:"
    echo "report_name is the name of the text file"
    echo "-today = only shots from today"
    echo "-i = rollname checking is case insensitive"
    echo "<rollname> = only include shots with this rollname"
    exit -1
fi

tidyFunction()
{
    if [ -f xml.lst ]; then
        rm -f xml.lst
    fi
    if [ -f codexxml.lst ]; then
        rm -f codexxml.lst
    fi
    if [ -f codexxml.tmp ]; then
        rm -f codexxml.tmp
    fi
}

REPORT_NAME=""
CASE_SENSITIVE_SEARCH=1
TODAY_SEARCH=0
ROLLNAME=""

for i in $@
do
    if [ ${#REPORT_NAME} -eq 0 ]; then
        REPORT_NAME=$i
    elif [ $i = "-i" ]; then
        CASE_SENSITIVE_SEARCH=0
    elif [ $i = "-today" ]; then
        TODAY_SEARCH=1
    elif [ ${#ROLLNAME} -eq 0 ]; then
        ROLLNAME=$i
        break
    fi
done

tidyFunction

# find xml files in order
echo "Searching for xml files on VFS..."
FIND_ARGS="/mnt/codexvfs"
#if [ $TODAY_SEARCH -eq 1 ];then
#   FIND_ARGS=$FIND_ARGS" -mtime 0"
#fi

find $FIND_ARGS -type f -print0 |
     xargs -0 ls -l --time-style=long-iso |
     grep .xml |
     awk '{ print $6" "$7" "$8 }' |
     sort -n 2> /dev/null |
     awk '{ print $3 }' > xml.lst

# only leave in codex xmls files
echo "Filtering out non Codex Shot type xml files..."
for i in `cat xml.lst`;
do
     grep -l "<CodexShotMetadata" $i >> codexxml.lst
done

# filter out roll if necessary
if [ ${#ROLLNAME} -ne 0 ]; then
    echo "Filtering out shots not on roll $ROLLNAME..."
    GREP_ARGS="-l"
    if [ $CASE_SENSITIVE_SEARCH -eq 0 ];then
        GREP_ARGS=$GREP_ARGS" -i"
    fi

   mv codexxml.lst codexxml.tmp
   for i in `cat codexxml.tmp`;
   do
       if grep $GREP_ARGS "$ROLLNAME" $i > /dev/null; then
           echo $i >> codexxml.lst
       fi
   done

   if [ ! -e codexxml.lst ]; then
       echo No shots found on roll $ROLLNAME
       exit -1
   fi
fi

# now go through list of codex xml files and pull out data of interest
echo "Now pulling out data of interest and formatting report..."

COUNT=1
NUM_FILES=`cat codexxml.lst | wc -l`
if [ $NUM_FILES -eq 0 ]; then
    echo "No matching xml files found."
    exit -1 
fi

echo "-----------------------------------------------------------------" > $REPORT_NAME.txt
echo "Camera report generated on `date`" >> $REPORT_NAME.txt
echo "-----------------------------------------------------------------" >> $REPORT_NAME.txt
echo "" >> $REPORT_NAME.txt

for i in "`cat codexxml.lst | head -1`";
do
   awk ' function getProdInfo()
         {
             FS=">";
             $0=$0; end=index($2,"<")-1; value=substr($2,1,end); return value;
         }
         /<ProductionCompany>/ { pc = getProdInfo(); }
         /<DirectorOfPhotography>/ { dp = getProdInfo(); }
         /<ProductionName>/ { pn = getProdInfo(); }
         END {
             printf("Production Company: %s\n", pc);
             printf("Production: %s\n", pn);
             printf("DoP: %s\n", dp);
         }

       ' $i >> $REPORT_NAME.txt
done

echo "" >> $REPORT_NAME.txt
echo "NOTES:" >> $REPORT_NAME.txt

if [ $TODAY_SEARCH -eq 1 ];then
    echo "* Limited to today's shots only" >> $REPORT_NAME.txt
else
    echo "* Searching all dates" >> $REPORT_NAME.txt
fi

if [ ${#ROLLNAME} -ne 0 ]; then
    if [ $CASE_SENSITIVE_SEARCH -eq 0 ];then
       echo "* Limited to roll $ROLLNAME (case insensitive)" >> $REPORT_NAME.txt
    else
       echo "* Limited to roll $ROLLNAME" >> $REPORT_NAME.txt
    fi
else
    echo "* Searching all rolls" >> $REPORT_NAME.txt 
fi

echo "* $NUM_FILES shot(s)" >> $REPORT_NAME.txt

echo "" >> $REPORT_NAME.txt
echo "-----------------------------------------------------------------" >> $REPORT_NAME.txt 
echo " | Shot |           | Camera Roll |     | Timecode |   | Frames | " >> $REPORT_NAME.txt 
echo "-----------------------------------------------------------------" >> $REPORT_NAME.txt 

TOTAL_FRAME_COUNT=0
for i in `cat codexxml.lst`; 
do
   printf "Progress: %d/%d\r" $COUNT $NUM_FILES
   awk ' BEGIN { minFrames = -1; firstSample = 1; }

         function getValue()
         { 
             FS=">"; 
             $0=$0; 
             end=index($2,"<")-1; 
             value=substr($2,1,end); 
             return value; 
         }
         /<ShotName>/       { name = getValue(); }
         /<OriginalRoll>/   { roll = getValue(); }
         /<Timecode/        { inTcSection = 1; }
         /Timecode>/        { inTcSection = 0; }
         /<Channel id="0">/ { if (inTcSection) tcValidChan = 1; }
         /Channel>/         { tcValidChan = 0; }
         /<StartTc>/        { if (tcValidChan) tc = getValue(); }

         /<Samples>/        { frames = getValue();
                              if (firstSample || (int(frames) < int(minFrames)))
                              {  
                                  firstSample = 0;
                                  minFrames = frames;
                              }
                            }

         END {
             printf("%-25s %-14s %-16s %-14d\n", name, roll, tc, minFrames); 
         }
       ' $i >> $REPORT_NAME.txt

    FRAME_COUNT=`grep -m 1 Samples $i | awk '{ x = substr($1, 10); printf "%d", x; }'`

    let TOTAL_FRAME_COUNT=$TOTAL_FRAME_COUNT+$FRAME_COUNT
    let COUNT=$COUNT+1

done

DPX_COUNT=`find /mnt/codexvfs -name "*.dpx" | wc -l`

echo >> $REPORT_NAME.txt
echo "Total frames reported in XML is $TOTAL_FRAME_COUNT" >> $REPORT_NAME.txt
echo "Number of DPX files found is $DPX_COUNT" >> $REPORT_NAME.txt

ARI_COUNT=`find /mnt/codexvfs -name "*.ari" | wc -l`
if [ $ARI_COUNT -gt 0 ]; then
   echo "Number of ARI files found is $ARI_COUNT" >> $REPORT_NAME.txt
fi

echo "Here is a copy of the report:"
echo
cat $REPORT_NAME.txt
echo 
echo "This report is stored as the file `pwd`/$REPORT_NAME.txt"

tidyFunction

