# listtapes.sh
# Searches .toc files in /etc/codex/tape-offloads and 
# sub directories and from this produces a list of what 
# rolls and shots are on what tape.
# These are in a form you can import into a spreadsheet
# fairly easily - each line is a shot showing roll, shot
# name and tape name which are space delimited.
# Limitations:
# Displays the contents of the latest use of a tape.  It
# may be that this last use failed - this is not yet
# checked (this information is stored in another file).
# If the TOC is pre version 2.0 (xml) then shot details
# are devined from the directory path and filename -
# this assumes that the form that the offload contains 
# dpx's and that the format in the vfs is:
# .dpx {r}/{e}/{r}_{s}_{u}/<anything>.{e}
# OR
# .dpx {r}/{s}/<anything>.{e}
# OR
# .dpx {r}/{s}/{s}_{u}_<anything>.{e}
# CR! 24/06/09

rm -f x
rm -f y
rm -f tape.list
rm -f badtape.list

TAPE_FOLDER=$1
if [ ${#TAPE_FOLDER} -eq 0 ]; then
    TAPE_FOLDER="/etc/codex/tape-offloads"
#    TAPE_FOLDER="/var/log"
fi

LOG_NAME=$2
if [ ${#LOG_NAME} -eq 0 ]; then
    LOG_NAME="progress.log"
#    LOG_NAME="offload.log"
fi

# find all toc files in reverse order - i.e.
# newest first - and store in file x
find $TAPE_FOLDER -type f -print0 |
    xargs -0 ls -l --time-style=long-iso |
    grep .toc |
    awk '{ print $6" "$7" "$8 }' |
    sort -nr 2> /dev/null |
    awk '{ print $3 }' > x

# scan list in file x output only the first (i.e. latest
# toc which would be the last use of the tape) to file y
awk ' /.toc/ { 
    toc = substr($0, length($0) - 11, 8);
    if (!index(list, toc))  
    { 
        print $0; 
        temp = list " " toc; 
        list = temp; 
    } 
}' x > y

# from file y list rollnames and shots
COUNT=1
NUM_FILES=`cat y | wc -l`
for I in `cat y`;
do
    printf "progress: %d/%d: %-60s\r" $COUNT $NUM_FILES $I
    let COUNT=$COUNT+1

    # check if there's was an error on the last outing for this
    # tape and if so ignore it
    # We extract the directory and the toc name and then see if
    # the progress log in that directory indicates tape is bad

    # Get the toc filename by removing from first / to last /
    # in filename
    TOC=${I##/*/}

    # Get dir by removing toc filename from end
    DIR=${I%$TOC}

    # remove file extension from toc filename to get tape name
    TOC=${TOC%\.toc}

    # now see if an error was reported using this tape in this
    # offload - res is the length of the return string from 
    # the find and grep - if its empty then its not bad
    TEST=`find $DIR -name $LOG_NAME -exec grep -iE "(bad tape|dodgy) $TOC" {} \;`
    RES=${#TEST}
    if [ $RES -eq 0 ]; then

        # last tape use was good - list it contents
        if grep "TOCVersion" $I > /dev/null; then
            awk 'BEGIN {
                    nextLineIsCurrentTape = 0; 
                    tape="";
                    gotRoll = 0;
                    gotName = 0;
                    gotTc = 0;
                    thisTape = 0;
                }
                /<ThisTape>/ {
                    nextLineIsCurrentTape = 1;
                }
                /<Tape>/ {
                    if (nextLineIsCurrentTape == 1)
                    {
                        tape = getValue();
                        nextLineIsCurrentTape = 0;
                    }
                    else
                    {
                        if (getValue() == tape)
                        {
                            thisTape = 1;
                            test();
                        }
                    }
                }
                /<ShotName>/ {
                    shotname = getValue();
                    gotName = 1;
                    test();
                }
                /RollName>/ {
                    rollname = getValue();
                    gotRoll = 1;
                    test();
                }
                /<StartTc/ {
                    tc = getValue();
                    gotTc = 1;
                    test();
                }

                function getValue()
                {
                    FS=">";
                    $0=$0;
                    end = index($2, "<") - 1;
                    value = substr($2, 0, end);
                    return value;
                }

                function test()
                {
                    if (thisTape==1 && gotName==1 && gotRoll==1 && gotTc==1)
                    {
                        printf("%s %s %s %s\n",
                                shotname, rollname, tc, tape);
                        gotName = 0;
                        gotRoll = 0;
                        gotTc = 0;
                        thisTape = 0;
                    }
                }
            ' $I >> tape.list
        else
            awk 'BEGIN{tape=""; line_count=0; cur_tape="null";} 
                /tape volume/ {
                    FS = " "; $0 = $0;
                    cur_tape = $5;
                    sub(/^[ \t\n\r]+/, "", cur_tape);
                    sub(/[ \t\n\r]+$/, "", cur_tape);
                }
                /This Tape:/ {
                    # remember current tape
                    FS = " "; $0 = $0;
                    tape = $3; 
                    sub(/^[ \t\n\r]+/, "", tape);
                    sub(/[ \t\n\r]+$/, "", tape);
                }
                /\.dpx|\.ari/ {
                    if (cur_tape == tape)
                    {
                        FS = "/"; $0 = $0;
                        roll = $1;
                        sub(/^[ \t\n\r]+/, "", roll);
                        start = index($3, "_");
                        if (start == 0)
                            name = $2;
                        else
                        {
                            start += 1;
                            name = substr($3, start, length($3) - start - 36); 
                            if (index($3, ".dpx") != 0)
                                name = substr($3, 1, start - 2);
                            else if (index($3, ".ari") != 0)
                                name = substr($3, 1, start - 2);
                        }
                        stuff = roll"&"name;
                        if (!index(list, stuff)) 
                        {
                            if (length(roll) && length(name) && length(tape))
                            {
                                printf("%s %s %s\n", name, roll, tape);
                                temp = list " " stuff; 
                                list = temp; 
                            }
                        }
                    }
                } 
            ' $I >> tape.list
        fi
    else
        echo "Tape $TOC is bad" >> badtape.list
    fi    
done

if [ -f tape.list ]; then
    echo "RESULTS (also written to tape.list)                                            "
    echo "==================================="
    echo ""
    cat tape.list
else
    echo "No valid tape contents were found.                                             "
fi

if [ -f badtape.list ]; then 
    echo ""
    cat badtape.list
fi


