#!/bin/sh

function tidy_up ()
{
    [ -f ./mtxstatus.err ] && rm ./mtxstatus.err
    [ -f ./mtxstatus.tmp ] && rm ./mtxstatus.tmp
    [ -f ./nst0status.err ] && rm ./nst0status.err
    [ -f ./nst0status.tmp ] && rm ./nst0status.tmp
    [ -f ./loadedtape.tmp ] && rm ./loadedtape.tmp
    [ -f ./loadedtape.err ] && rm ./loadedtape.err
    [ -f ./tapelist.tmp ] && rm ./tapelist.tmp
    [ -f ./tapelist.err ] && rm ./tapelist.err
}

function check_tape () 
{
    # check if tape already been done
    if [ ! -s ./tape_list.log ]; then
        return 1
    fi

    while read LINE
    do
        if [ "$LINE" == "$1" ]; then
            return 0
        fi
    done < ./tape_list.log
    return 1
}

tidy_up

echo "Checking for tape changer..."
# do twice to help prevent mtx autoloader status error issues
/usr/sbin/mtx status 1> ./mtxstatus.tmp 2> ./mtxstatus.err
/usr/sbin/mtx status 1> ./mtxstatus.tmp 2> ./mtxstatus.err
/bin/mt -f /dev/nst0 status 1> ./nst0status.tmp 2> ./nst0status.err
if [ -s ./mtxstatus.err ]; then

    # no tape changer
    echo "Error talking to tape changer:"
    echo "======="
    cat ./mtxstatus.err
    echo "======="
    echo "Assuming single tape drive(s)"

    # just grab from current inserted tape?
    if [ -s ./nst0status.err ]; then
        echo "ERROR: status request for nst0 failed:"
        echo "======="
        cat ./nst0status.err
        echo "======="
        exit -1
    fi
    if grep "DR_OPEN" ./nst0status.tmp; then
        echo "Dumping from tape in nst0"
        /root/dumptape.sh
    else
        echo "ERROR: No tapes found in any drive nst0"
        exit -1
    fi
else
    # all good
    NUM_DRIVES=`grep "Data Transfer Element" ./mtxstatus.tmp | wc -l`
    if [ $NUM_DRIVES -eq 0 ]; then
        echo "ERROR: Tape changer found but no drives"
        exit -1
    fi
    echo "Tape changer found: $NUM_DRIVES drives"
    if [ $NUM_DRIVES -gt 1 ]; then
        echo "N.B. Only using 1 drive"
    fi

    # dump currently loaded tape if there is one
    CUR_SLOT=0
    TAPE_COUNT=1
    grep Data.*Full.*VolumeTag ./mtxstatus.tmp 1> ./loadedtape.tmp 2> ./loadedtape.err
    grep Storage.*Full.*VolumeTag ./mtxstatus.tmp 1> ./tapelist.tmp 2> ./tapelist.err
    NUM_TAPES=`cat ./tapelist.tmp | wc -l`
    if [ -s ./loadedtape.tmp ]; then

        let NUM_TAPES=$NUM_TAPES+1
        CUR_TAPE=`awk '{ print $10 }' ./loadedtape.tmp`
        CUR_SLOT=`awk '{ print $7 }' ./loadedtape.tmp`

        # it is possible that a loaded tape can return an unknown
        # slot as its storage slot - if this is the case then find
        # an empty slot to store it in
        if [[ "$CUR_SLOT" =~ [^0-9]+$ ]]; then
            echo "Currently loaded tape has no home slot."
            echo "Trying to find empty slot"
            CUR_SLOT=`/usr/sbin/mtx status | grep Storage.Element.*:Empty: | awk 'NR == 1 { test=$3+0; print test }'`
            if [ "$CUR_SLOT" == "" ]; then
                echo "No spare slots to house current tape in."
                exit -1
            fi
            echo "$CUR_TAPE's home slot is now $CUR_SLOT"
        fi

        check_tape $CUR_TAPE
        if [ $? -eq 1 ]; then
            echo "Dumping from already loaded tape $CUR_TAPE ($TAPE_COUNT of $NUM_TAPES)"
            /root/dumptape.sh $CUR_TAPE
        else
            echo "Skipping tape $CUR_TAPE - already unarchived"
        fi
        let TAPE_COUNT=$TAPE_COUNT+1
    fi

    # go through other tapes
    while read LINE; do

        if [ $CUR_SLOT -gt 0 ]; then
            echo "Unloading current tape $CUR_TAPE"
            /usr/sbin/mtx unload $CUR_SLOT 0
        fi
        
        CUR_SLOT=${LINE%:Full*}
        CUR_SLOT=${CUR_SLOT:16}
        CUR_TAPE=${LINE:(-8)}
        check_tape $CUR_TAPE 
        if [ $? -eq 1 ]; then
            echo "Loading tape $CUR_TAPE"
            /usr/sbin/mtx load $CUR_SLOT 0
            echo "Dumping tape $CUR_TAPE ($TAPE_COUNT of $NUM_TAPES)"
            /root/dumptape.sh $CUR_TAPE
        else
            echo "Skipping tape $CUR_TAPE - already unarchived"
            CUR_SLOT=0
        fi

        let TAPE_COUNT=$TAPE_COUNT+1
    done < ./tapelist.tmp
fi

tidy_up



