/*
**  @(#) 1.7 src/ltfs.h, LTFS, ltfs.01.02.00, 201102171 2/15/11 02:23:06
**
**  ZZ_Copyright_BEGIN
**  
**  
**  Licensed Materials - Property of IBM
**  
**  IBM Linear Tape File System Single Drive Edition
**  
**  Copyright IBM Corp. 2010, 2011
**  
**  This file is part of the IBM Linear Tape File System Single Drive Edition
**  (formally known as IBM Long Term File System)
**  
**  The IBM Linear Tape File System Single Drive Edition is free software;
**  you can redistribute it and/or modify it under the terms of the GNU Lesser 
**  General Public License as published by the Free Software Foundation, 
**  version 2.1 of the License.
**  
**  The IBM Linear Tape File system Single Drive Edition is distributed in the 
**  hope that it will be useful, but WITHOUT ANY WARRANTY; without even the 
**  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**  See the GNU Lesser General Public License for more details.
**  
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
**  or download the license from <http://www.gnu.org/licenses/>.
**  
**  
**  ZZ_Copyright_END
**
*************************************************************************************
**
** COMPONENT NAME:  IBM Linear Tape File System
**
** FILE NAME:       ltfs.h
**
** DESCRIPTION:     Defines LTFS data structures and prototypes for libltfs.
**
** AUTHORS:         Brian Biskeborn
**                  IBM Almaden Research Center
**                  bbiskebo@us.ibm.com
**
**                  Lucas C. Villa Real
**                  IBM Almaden Research Center
**                  lucasvr@us.ibm.com
**
*************************************************************************************
**
**  (c) Copyright 2010 Hewlett-Packard Development Company, L.P.
**  06/10/10 Define PACKAGE_OWNER and reference it in ltfs.c to make it configurable
**           Replace PACKAGE_VERSION with HP nomenclature
**
*************************************************************************************
*/
#ifndef __ltfs_h__
#define __ltfs_h__

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include <stdbool.h>
#include <stddef.h>
#include <stdlib.h>
#include <inttypes.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <ctype.h>
#include <string.h>
#include <pthread.h>
#include <limits.h>
#include <time.h>
#include "queue.h" /* Must be included before <sys/mount.h> */
#ifndef mingw_PLATFORM
#include <termios.h>
#endif
#include <sys/time.h>
#include <sys/types.h>
#ifndef mingw_PLATFORM
#include <sys/mount.h>
#endif
#include <sys/stat.h>

#ifdef __APPLE__
#include <ICU/unicode/utypes.h>
#else
#include <unicode/utypes.h>
#endif

#ifndef mingw_PLATFORM
#include <sys/xattr.h>
#endif
#ifndef ENOATTR
#define ENOATTR ENODATA  /* Attribute not found */
#endif /* ENOATTR */

#ifdef __APPLE__
#include "arch/osx/osx_errno.h"
#endif

#include "ltfs_error.h"
#include "ltfsprintf.h"
#include "ltfs_types.h"
#include "mrsw.h"

/* forward declarations from tape.h and tape_ops.h */
struct tape_ops;
struct device_data;

#ifdef mingw_PLATFORM
#define LTFS_DEFAULT_WORK_DIR         "c:/tmp/ltfs"
#else
#define LTFS_DEFAULT_WORK_DIR         "/tmp/ltfs"
#endif

#define LTFS_NUM_PARTITIONS           2
#define LTFS_FILENAME_MAX             255
#define LTFS_MAX_XATTR_SIZE           4096

#define LTFS_SUPER_MAGIC              0x7af3
#define LTFS_DEFAULT_BLOCKSIZE        (512*1024)

#define MAKE_LTFS_VERSION(x,y,z)      (10000*(x) + 100*(y) + (z))

#define LTFS_LABEL_VERSION_MIN        MAKE_LTFS_VERSION(1,0,0)   /* Min supported label version */
#define LTFS_LABEL_VERSION_MAX        MAKE_LTFS_VERSION(2,99,99) /* Max supported label version */
#define LTFS_LABEL_VERSION            MAKE_LTFS_VERSION(2,0,0)   /* Written label version */
#define LTFS_LABEL_VERSION_STR        "2.0.0"    /* Label version string */

#define LTFS_INDEX_VERSION_MIN        MAKE_LTFS_VERSION(1,0,0)    /* Min supported index version */
#define LTFS_INDEX_VERSION_MAX        MAKE_LTFS_VERSION(2,99,99)  /* Max supported index version */
#define LTFS_INDEX_VERSION            MAKE_LTFS_VERSION(2,0,0)    /* Written index version */
#define LTFS_INDEX_VERSION_STR        "2.0.0"  /* Index version string */

#define INDEX_MAX_COMMENT_LEN         65536 /* Maximum comment field length (per LTFS Format) */

#ifdef __APPLE__
#define PACKAGE_NAME                  "LTFS"
#define PACKAGE_VERSION               "1.2.0"
#else
#include "config.h"
#endif

#ifdef HP_BUILD
#define PACKAGE_OWNER                 "HP LTFS"
#elif defined QUANTUM_BUILD
#define PACKAGE_OWNER                 "Quantum LTFS"
#else
#define PACKAGE_OWNER                 "IBM LTFS"
#endif
#define LTFS_LOSTANDFOUND_DIR         "_ltfs_lostandfound"

/**
 * All capacities are relative to filesystem block size.
 */
struct device_capacity {
    unsigned long remaining_ip; /**< Remaining capacity of index partition */
    unsigned long remaining_dp; /**< Remaining capacity of data partition */
    unsigned long total_ip;     /**< Total capacity of index partition */
    unsigned long total_dp;     /**< Total capacity of data partition */
};

struct tape_offset {
	tape_block_t block;
	char partition;
};

/**
 * Extent information
 */
struct extent_info {
	TAILQ_ENTRY(extent_info) list;
	struct tape_offset start;
	uint32_t byteoffset;
	uint64_t bytecount;
	uint64_t fileoffset;
};

/**
 * Extended attributes
 */
struct xattr_info {
	TAILQ_ENTRY(xattr_info) list;
	char  *key;
	char  *value;
	size_t size;
};

struct dentry {
	TAILQ_ENTRY(dentry) myself;    /**< Part of the linked list from parent's children list */
	TAILQ_HEAD(child_struct, dentry) list;             /**< List of children */
	TAILQ_HEAD(extent_struct, extent_info) extentlist; /**< List of extents */
	TAILQ_HEAD(xattr_struct, xattr_info) xattrlist;    /**< List of extended attributes */
	struct dentry *parent;                             /**< Pointer to parent dentry */
	struct MultiReaderSingleWriter lock;               /**< Lock for dentry modifications */
	ino_t    ino;                  /**< Per-session inode number, unique across all LTFS volumes in this process */
	uint64_t uid;                  /**< Persistent unique ID */
	uint64_t realsize;             /**< Size, not counting sparse tail */
	uint64_t size;                 /**< File size */
	bool     isdir;                /**< True if this is a directory, false if it's a file */
	bool     readonly;             /**< True if file is marked read-only */
	struct timespec creation_time; /**< Time of creation */
	struct timespec modify_time;   /**< Time of last modification */
	struct timespec access_time;   /**< Time of last access */
	struct timespec change_time;   /**< Time of last status change */
	struct timespec backup_time;   /**< Time of last backup */
	char    *name;                 /**< File or directory name */
	char    *platform_safe_name;   /**< File or directory name after file name mangling */
	uint32_t numhandles;           /**< Reference count */
	bool     deleted;              /**< True if dentry is unlinked from the file system */
	bool matches_name_criteria;    /**< True if file name matches the name criteria rules */
	pthread_mutex_t iosched_lock;  /**< Lock for use by the I/O scheduler */
	void *iosched_priv;            /**< I/O scheduler private data */
	size_t tag_count;              /**< Number of unknown tags */
	unsigned char **preserved_tags; /**< Unknown tags to be preserved on tape */
};

struct ltfs_volume {
	char *creator;                 /**< Creator string to use when writing labels, index files */
	struct ltfs_label *label;      /**< Information from the partition labels */
	struct ltfs_index *index;      /**< Current cartridge index */
	struct device_data *device;    /**< Device-specific data */
	void *iosched_handle;          /**< Handle to the I/O scheduler state */

	struct tc_coherency ip_coh;    /**< Index partition coherency info */
	struct tc_coherency dp_coh;    /**< Data partition coherency info */

	bool ip_index_file_end;        /**< Does the index partition end in an index file? */
	bool dp_index_file_end;        /**< Does the data partition end in an index file? */

	/* A 1-block read cache, used to prevent reading the same block from tape over and over.
	 * You MUST hold the tape device lock before accessing this buffer. */
	struct tape_offset last_pos;   /**< Position of last block read from the tape. */
	unsigned long last_size;       /**< Size of last block read from the tape. */
	char *last_block;              /**< Contents of last block read from the tape. */

	char *work_directory;          /**< Temporary directory for index files */
	void *opt_args;                /**< FUSE command-line arguments */
	bool syncfs_thread_keepalive;  /**< Used to terminate the ltfs_syncfs_thread */
	bool skip_eod_check;           /**< Skip EOD existance check? */
	bool ignore_wrong_version;     /**< Ignore wrong index version while seeking index? */
};

struct ltfs_label {
	char *creator;                 /**< Program that wrote this label */
	int version;                   /**< Label format version, as formatted by MAKE_LTFS_VERSION */
	char tape_serial[7];           /**< Tape volume serial (up to 6 alphanumeric ASCII chars) */
	char vol_uuid[37];             /**< LTFS volume UUID */
	struct timespec format_time;   /**< time when this volume was formatted */
	unsigned long blocksize;       /**< Preferred tape blocksize */
	bool enable_compression;       /**< Enable data compression on tape */

	/* physical <-> logical partition mapping */
	char this_partition;           /**< Logical ID of this partition (used on read) */
	char partid_dp;                /**< Logical ID of data partition */
	char partid_ip;                /**< Logical ID of index partition */
	char part_num2id[LTFS_NUM_PARTITIONS]; /**< Mapping physical partition -> logical ID */
};

/**
 * Index partition criteria.
 *
 * The high and low water mark define how many objects of size @max_filesize_criteria
 * to allocate at initialization and at most for a given session. The filename_criteria
 * array defines glob patterns that are looked at by the file creation routines so that
 * the I/O scheduler can know if the new file is a candidate to go to the index partition
 * or not.
 *
 * Please note that when @max_filesize_criteria is 0 then no caching is performed by
 * LTFS and all files go straight to the data partition. The index partition only use
 * in this case is to store indices.
 */
struct index_criteria {
	bool have_criteria;              /**< Does this struct actually specify criteria? */
	uint64_t max_filesize_criteria;  /**< Maximum file size that goes into the index partition */
	size_t low_water_mark;           /**< How many cache objects to allocate at mount time */
	size_t high_water_mark;          /**< How many cache objects to keep at most */
	char **glob_patterns;            /**< NULL-terminated list of file name criteria */
	UChar **glob_cache;              /**< Cache of glob patterns in comparison-ready form */
};

struct ltfs_index {
	/* acquire this lock for read before using the index. acquire it for write before
	 * writing the index to tape. */
	struct MultiReaderSingleWriter lock;  /**< Controls access to index metadata */

	char *creator;                 /**< Program that wrote this index */
	char vol_uuid[37];             /**< LTFS volume UUID */
	char *volume_name;             /**< human-readable volume name */
	unsigned int generation;       /**< last generation number written to tape */
	struct timespec mod_time;      /**< time of last modification */
	struct tape_offset selfptr;    /**< self-pointer (where this index was recovered from tape) */
	struct tape_offset backptr;    /**< back pointer (to prior generation on data partition) */

	bool criteria_allow_update;    /**< Can the index criteria be changed? */
	struct index_criteria original_criteria; /**< Index partition criteria from the medium */
	struct index_criteria index_criteria; /**< Active index criteria. May contain per-session settings */

	struct dentry *root;           /**< The directory tree */

	/* Update tracking */
	pthread_mutex_t dirty_lock;    /**< Controls access to the update tracking bits */
	bool dirty;                    /**< Set on metadata update, cleared on write to tape */
	bool atime_dirty;              /**< Set on atime update, cleared on write to tape */
	bool use_atime;                /**< Set if atime updates should make the index dirty */
	uint64_t file_count;           /**< Number of files in the file system */
	uint64_t uid_number;           /**< File/directory's most recently reserved uid number */
	char *commit_message;          /**< Commit message specified by the "user.ltfs.sync" xattr */
	int version;                   /**< Index format version, as formatted by MAKE_LTFS_VERSION */

	size_t tag_count;               /**< Number of unrecognized tags */
	unsigned char **preserved_tags; /**< Unrecognized tags, will be preserved when writing tape */
};

/* Definitions for sync */
typedef enum {
	LTFS_SYNC_TIME,
	LTFS_SYNC_CLOSE,
	LTFS_SYNC_UNMOUNT
} ltfs_sync_type_t;

#define LTFS_SYNC_PERIOD_DEFAULT (5 * 60) /* default sync period (5 minutes) */

#define WRITE_LOCK(dentry) do { \
	acquirewrite_mrsw(&(dentry)->lock); \
	if ((dentry)->parent) \
		acquirewrite_mrsw(&(dentry)->parent->lock); \
} while(0)

#define WRITE_UNLOCK(dentry) do { \
	if ((dentry)->parent) \
		releasewrite_mrsw(&(dentry)->parent->lock); \
	releasewrite_mrsw(&(dentry)->lock); \
} while(0)

#define READ_LOCK(dentry) do { \
	acquireread_mrsw(&(dentry)->lock); \
	if ((dentry)->parent) \
		acquireread_mrsw(&(dentry)->parent->lock); \
} while(0)

#define READ_UNLOCK(dentry) do { \
	if ((dentry)->parent) \
		releaseread_mrsw(&(dentry)->parent->lock); \
	releaseread_mrsw(&(dentry)->lock); \
} while(0)

const char *ltfs_version();
const char *ltfs_format_version();
int ltfs_init(int log_level, bool use_syslog);
void ltfs_set_log_level(int log_level);
int ltfs_finish();

/* Public wrappers for tape_* functions */
const char *ltfs_default_device_name(struct tape_ops *ops);
int ltfs_device_open(const char *devname, struct tape_ops *ops, struct ltfs_volume *vol);
void ltfs_device_close(struct ltfs_volume *vol);
int ltfs_setup_device(struct ltfs_volume *vol);
int ltfs_get_tape_readonly(struct ltfs_volume *vol);

int ltfs_volume_alloc(const char *execname, struct ltfs_volume **volume);
void ltfs_volume_free(struct ltfs_volume **volume);
int ltfs_capacity_data(struct device_capacity *cap, struct ltfs_volume *vol);
int ltfs_mount(bool force_full, bool deep_recovery, bool recover_extra, struct ltfs_volume *vol);
int ltfs_unmount(struct ltfs_volume *vol);
int ltfs_eject_tape(struct ltfs_volume *vol);
int ltfs_write_label(tape_partition_t partition, struct ltfs_volume *vol);
int ltfs_format_tape(struct ltfs_volume *vol, bool force);
int ltfs_recover_eod(struct ltfs_volume *vol);
void ltfs_recover_eod_simple(struct ltfs_volume *vol);

void ltfs_use_atime(bool use_atime, struct ltfs_volume *vol);
void ltfs_set_workdir(const char *workdir, struct ltfs_volume *vol);

void ltfs_set_index_dirty(bool locking, bool atime, struct ltfs_index *idx);
void ltfs_unset_index_dirty(bool update_version, struct ltfs_index *idx);

int ltfs_open(const char *path, int flags, struct dentry **dentry, struct ltfs_volume *vol);
int ltfs_close(struct dentry *d, struct ltfs_volume *vol);
int ltfs_create(const char *path, bool isdir, bool readonly, struct dentry **dentry,
	struct ltfs_volume *vol);
int ltfs_unlink(const char *path, struct ltfs_volume *vol);
int ltfs_parse_backend_opts(void *opt_args, struct ltfs_volume *vol);

int ltfs_getattr(const char *path, struct dentry *dentry, struct stat *stbuf,
	struct ltfs_volume *vol);
int ltfs_utimens(const char *path, const struct timespec ts[2], struct ltfs_volume *vol);
int ltfs_set_readonly(const char *path, struct dentry *dentry, bool readonly,
	struct ltfs_volume *vol);
int ltfs_rename(const char *from, const char *to, struct ltfs_volume *vol);

int ltfs_truncate(struct dentry *d, off_t length, struct ltfs_volume *vol);
ssize_t ltfs_write(struct dentry *d, const char *buf, size_t count, off_t offset,
	char partition, struct ltfs_volume *vol);
ssize_t ltfs_write_noupdate(struct dentry *d, const char *buf, size_t count, off_t offset,
	char partition, struct extent_info **extent, bool update_time, struct ltfs_volume *vol);
int ltfs_write_index(char partition, bool locking, struct ltfs_volume *vol);
ssize_t ltfs_read(struct dentry *d, char *buf, size_t count, off_t offset, struct ltfs_volume *vol);
int ltfs_append_extent(struct dentry *d, struct extent_info *ext, bool update_time,
	struct ltfs_volume *vol);
int ltfs_append_extent_unlocked(struct dentry *d, struct extent_info *ext, bool update_time,
	struct ltfs_volume *vol);
int ltfs_write_data(char partition, const char *buf, size_t count, uint64_t repetitions,
	tape_block_t *startblock, bool locking, struct ltfs_volume *vol);

char ltfs_dp_id(struct ltfs_volume *vol);
char ltfs_ip_id(struct ltfs_volume *vol);

struct dentry *ltfs_get_dentry(struct dentry *d, struct ltfs_volume *vol);
void ltfs_put_dentry(struct dentry *d, struct ltfs_volume *vol);

int ltfs_sync_index(struct ltfs_volume *vol, bool lock);
int ltfs_check_eod_status(struct ltfs_volume *vol);

void ltfs_print_help_message(struct tape_ops *ops);

#ifdef __cplusplus
}
#endif

#endif /* __ltfs_h__ */
