/*
**  @(#) 1.2 src/ltfsprintf.h, LTFS, ltfs.01.02.00, 201102171 2/15/11 02:23:10
**
**  ZZ_Copyright_BEGIN
**  
**  
**  Licensed Materials - Property of IBM
**  
**  IBM Linear Tape File System Single Drive Edition
**  
**  Copyright IBM Corp. 2010, 2011
**  
**  This file is part of the IBM Linear Tape File System Single Drive Edition
**  (formally known as IBM Long Term File System)
**  
**  The IBM Linear Tape File System Single Drive Edition is free software;
**  you can redistribute it and/or modify it under the terms of the GNU Lesser 
**  General Public License as published by the Free Software Foundation, 
**  version 2.1 of the License.
**  
**  The IBM Linear Tape File system Single Drive Edition is distributed in the 
**  hope that it will be useful, but WITHOUT ANY WARRANTY; without even the 
**  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**  See the GNU Lesser General Public License for more details.
**  
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
**  or download the license from <http://www.gnu.org/licenses/>.
**  
**  
**  ZZ_Copyright_END
*/

#ifndef __ltfs_printf_h
#define __ltfs_printf_h

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include <stdbool.h>
#include <inttypes.h>

enum ltfs_log_levels {
	LTFS_ERR   = 0, /* Fatal error or operation failed unexpectedly */
	LTFS_WARN  = 1, /* Unexpected condition, but the program can continue */
	LTFS_INFO  = 2, /* Helpful message */
	LTFS_DEBUG = 3, /* Diagnostic messages */
	LTFS_TRACE = 4, /* Full call tracing */
};

extern int ltfs_verbosity; /* TODO: get rid of me */
extern int ltfs_log_level;

/* Old macros. TODO: get rid of these. */
#define ltfslog(level, fmt, ...)					\
	do { \
		if (level <= ltfs_verbosity) \
			fprintf(stderr, "[verbose%d] "fmt, level, ##__VA_ARGS__); \
	} while(0)

#define ltfsprintf(fmt, ...)						\
	do { \
		fprintf(stderr, "["__FILE__"::%d] "fmt"\n", __LINE__, ##__VA_ARGS__);	\
	} while(0)

/* Put this macro in front of a function definition to prevent gcc from tracing it. */
#ifdef mingw_PLATFORM
#define NOTRACE_FN
#else
#define NOTRACE_FN __attribute__((no_instrument_function))
#endif

/* Wrapper for ltfsmsg_internal. It only invokes the message print function if the requested
 * log level is not too verbose. */
#define ltfsmsg(level, id, ...) \
	do { \
		if (level <= ltfs_log_level) \
			ltfsmsg_internal(true, level, id, ##__VA_ARGS__); \
	} while (0)

/* Wrapper for ltfsmsg_internal that prints a message without the LTFSnnnnn prefix. It
 * always invokes the message print function, regardless of the message level. */
#define ltfsresult(id, ...) \
	do { \
		ltfsmsg_internal(false, LTFS_TRACE + 1, id, ##__VA_ARGS__); \
	} while (0)

/* Shortcut for asserting that a function argument is not NULL. It generates an error-level
 * message if the given argument is NULL. Functions for which a NULL argument is a warning
 * should not use this macro. */
#define CHECK_ARG_NULL(var, ret) \
	do { \
		if (! (var)) { \
			ltfsmsg(LTFS_ERR, "10005E", #var, __FUNCTION__); \
			return ret; \
		} \
	} while (0)

/**
 * Initialize the logging and error reporting functions.
 * @param log_level Logging level (generally one of LTFS_ERROR...LTFS_TRACE).
 * @param use_syslog Send error/warning/info messages to syslog? This function does not call
 *                   openlog(); the calling application may do so if it wants to.
 * @return 0 on success or a negative value on error.
 */
int ltfsprintf_init(int log_level, bool use_syslog);

/**
 * Tear down the logging and error reporting framework.
 */
void ltfsprintf_finish();

/**
 * Load messages for a plugin from the specified resource name.
 * @param bundle_name Message bundle name.
 * @param bundle_data Message bundle data structure.
 * @param messages On success, contains a handle to the loaded message bundles. That handle
 *                 should be passed to @ltfsprintf_unload_plugin later.
 * @return 0 on success or a negative value on error.
 */
int ltfsprintf_load_plugin(const char *bundle_name, void *bundle_data, void **messages);

/**
 * Stop using messages from the given plugin message bundle.
 * @param handle Message bundle handle, as returned by @ltfsprintf_load_plugin.
 */
void ltfsprintf_unload_plugin(void *handle);

/**
 * Print a message in the system locale. Any extra arguments are substituted into the
 * format string. The current logging level is ignored, so the ltfsmsg macro
 * (which calls this function) should be used instead.
 * The generated output goes to stderr. If syslog is enabled, messages of severity LTFS_INFO
 * through LTFS_ERR go to syslog as well. LTFS_DEBUG and LTFS_TRACE level messages always go
 * only to stderr.
 * @param print_id Print the message prefix LTFSnnnnn ?
 * @param level Log level of this message, must be one of the ltfs_log_levels (LTFS_ERROR, etc.).
 * @param id Unique ID of this error.
 * @return 0 if a message was printed or a negative value on error.
 */
NOTRACE_FN int ltfsmsg_internal(bool print_id, int level, const char *id, ...);

#ifdef __cplusplus
}
#endif

#endif /* __ltfs_printf_h */
